@@ -23,19 +23,68 @@
 
 namespace QuantLib {
 
-    QL_DEPRECATED_DISABLE_WARNING
+    InflationTermStructure::InflationTermStructure(
+                                        Date baseDate,
+                                        Frequency frequency,
+                                        const DayCounter& dayCounter,
+                                        ext::shared_ptr<Seasonality> seasonality,
+                                        Rate baseRate)
+    : TermStructure(dayCounter), seasonality_(std::move(seasonality)),
+      frequency_(frequency), baseRate_(baseRate), baseDate_(baseDate),
+      hasExplicitBaseDate_(true) {
+        if (seasonality_ != nullptr) {
+            QL_REQUIRE(seasonality_->isConsistent(*this),
+                       "Seasonality inconsistent with inflation term structure");
+        }
+    }
+
+    InflationTermStructure::InflationTermStructure(
+                                        const Date& referenceDate,
+                                        Date baseDate,
+                                        Frequency frequency,
+                                        const DayCounter& dayCounter,
+                                        ext::shared_ptr<Seasonality> seasonality,
+                                        Rate baseRate)
+    : TermStructure(referenceDate, Calendar(), dayCounter), seasonality_(std::move(seasonality)),
+      frequency_(frequency), baseRate_(baseRate), baseDate_(baseDate),
+      hasExplicitBaseDate_(true) {
+        if (seasonality_ != nullptr) {
+            QL_REQUIRE(seasonality_->isConsistent(*this),
+                       "Seasonality inconsistent with inflation term structure");
+        }
+    }
+
+    InflationTermStructure::InflationTermStructure(
+                                        Natural settlementDays,
+                                        const Calendar& calendar,
+                                        Date baseDate,
+                                        Frequency frequency,
+                                        const DayCounter& dayCounter,
+                                        ext::shared_ptr<Seasonality> seasonality,
+                                        Rate baseRate)
+    : TermStructure(settlementDays, calendar, dayCounter), seasonality_(std::move(seasonality)),
+      frequency_(frequency), baseRate_(baseRate), baseDate_(baseDate),
+      hasExplicitBaseDate_(true) {
+        if (seasonality_ != nullptr) {
+            QL_REQUIRE(seasonality_->isConsistent(*this),
+                       "Seasonality inconsistent with inflation term structure");
+        }
+    }
 
     InflationTermStructure::InflationTermStructure(
                                         Rate baseRate,
                                         const Period& observationLag,
                                         Frequency frequency,
-                                        bool indexIsInterpolated,
                                         const DayCounter& dayCounter,
-                                        const ext::shared_ptr<Seasonality> &seasonality)
-    : TermStructure(dayCounter),
-      observationLag_(observationLag), frequency_(frequency), indexIsInterpolated_(indexIsInterpolated),
-      baseRate_(baseRate) {
-        setSeasonality(seasonality);
+                                        ext::shared_ptr<Seasonality> seasonality)
+    : TermStructure(dayCounter), seasonality_(std::move(seasonality)),
+      observationLag_(observationLag), frequency_(frequency),
+      baseRate_(baseRate),
+      hasExplicitBaseDate_(false) {
+        if (seasonality_ != nullptr) {
+            QL_REQUIRE(seasonality_->isConsistent(*this),
+                       "Seasonality inconsistent with inflation term structure");
+        }
     }
 
     InflationTermStructure::InflationTermStructure(
@@ -43,15 +92,16 @@ namespace QuantLib {
                                         Rate baseRate,
                                         const Period& observationLag,
                                         Frequency frequency,
-                                        const bool indexIsInterpolated,
                                         const Calendar& calendar,
                                         const DayCounter& dayCounter,
-                                        const ext::shared_ptr<Seasonality> &seasonality)
-    : TermStructure(referenceDate, calendar, dayCounter),
-      observationLag_(observationLag),
-      frequency_(frequency), indexIsInterpolated_(indexIsInterpolated),
-      baseRate_(baseRate) {
-        setSeasonality(seasonality);
+                                        ext::shared_ptr<Seasonality> seasonality)
+    : TermStructure(referenceDate, calendar, dayCounter), seasonality_(std::move(seasonality)),
+      observationLag_(observationLag), frequency_(frequency), baseRate_(baseRate),
+      hasExplicitBaseDate_(false) {
+        if (seasonality_ != nullptr) {
+            QL_REQUIRE(seasonality_->isConsistent(*this),
+                       "Seasonality inconsistent with inflation term structure");
+        }
     }
 
     InflationTermStructure::InflationTermStructure(
@@ -60,17 +110,25 @@ namespace QuantLib {
                                         Rate baseRate,
                                         const Period& observationLag,
                                         Frequency frequency,
-                                        bool indexIsInterpolated,
                                         const DayCounter &dayCounter,
-                                        const ext::shared_ptr<Seasonality> &seasonality)
-    : TermStructure(settlementDays, calendar, dayCounter),
-      observationLag_(observationLag),
-      frequency_(frequency), indexIsInterpolated_(indexIsInterpolated),
-      baseRate_(baseRate) {
-        setSeasonality(seasonality);
+                                        ext::shared_ptr<Seasonality> seasonality)
+    : TermStructure(settlementDays, calendar, dayCounter), seasonality_(std::move(seasonality)),
+      observationLag_(observationLag), frequency_(frequency),
+      baseRate_(baseRate),
+      hasExplicitBaseDate_(false) {
+        if (seasonality_ != nullptr) {
+            QL_REQUIRE(seasonality_->isConsistent(*this),
+                       "Seasonality inconsistent with inflation term structure");
+        }
     }
 
-    QL_DEPRECATED_ENABLE_WARNING
+    Date InflationTermStructure::baseDate() const {
+        if (hasExplicitBaseDate()) {
+            return baseDate_;
+        } else {
+            return inflationPeriod(referenceDate() - observationLag(), frequency()).first;
+        }
+    }
 
     void InflationTermStructure::setSeasonality(
                           const ext::shared_ptr<Seasonality>& seasonality) {
@@ -78,10 +136,9 @@ namespace QuantLib {
         seasonality_ = seasonality;
         if (seasonality_ != nullptr) {
             QL_REQUIRE(seasonality_->isConsistent(*this),
-                       "Seasonality inconsistent with "
-                       "inflation term structure");
+                       "Seasonality inconsistent with inflation term structure");
         }
-        notifyObservers();
+        update();
     }
 
 
@@ -104,16 +161,40 @@ namespace QuantLib {
     }
 
 
+    ZeroInflationTermStructure::ZeroInflationTermStructure(
+                                   Date baseDate,
+                                   Frequency frequency,
+                                   const DayCounter& dayCounter,
+                                   const ext::shared_ptr<Seasonality>& seasonality)
+    : InflationTermStructure(baseDate, frequency, dayCounter, seasonality) {}
+
+    ZeroInflationTermStructure::ZeroInflationTermStructure(
+                                   const Date& referenceDate,
+                                   Date baseDate,
+                                   Frequency frequency,
+                                   const DayCounter& dayCounter,
+                                   const ext::shared_ptr<Seasonality>& seasonality)
+    : InflationTermStructure(referenceDate, baseDate, frequency, dayCounter, seasonality) {}
+
+    ZeroInflationTermStructure::ZeroInflationTermStructure(
+                                   Natural settlementDays,
+                                   const Calendar& calendar,
+                                   Date baseDate,
+                                   Frequency frequency,
+                                   const DayCounter& dayCounter,
+                                   const ext::shared_ptr<Seasonality>& seasonality)
+    : InflationTermStructure(settlementDays, calendar, baseDate, frequency, dayCounter, seasonality) {}
+
+    QL_DEPRECATED_DISABLE_WARNING
+
     ZeroInflationTermStructure::ZeroInflationTermStructure(
                                     const DayCounter& dayCounter,
                                     Rate baseZeroRate,
                                     const Period& observationLag,
                                     Frequency frequency,
-                                    bool indexIsInterpolated,
                                     const ext::shared_ptr<Seasonality> &seasonality)
-    : InflationTermStructure(baseZeroRate, observationLag, frequency, indexIsInterpolated,
-                             dayCounter, seasonality) {
-    }
+    : InflationTermStructure(baseZeroRate, observationLag, frequency,
+                             dayCounter, seasonality) {}
 
     ZeroInflationTermStructure::ZeroInflationTermStructure(
                                     const Date& referenceDate,
@@ -122,11 +203,9 @@ namespace QuantLib {
                                     Rate baseZeroRate,
                                     const Period& observationLag,
                                     Frequency frequency,
-                                    bool indexIsInterpolated,
                                     const ext::shared_ptr<Seasonality> &seasonality)
-    : InflationTermStructure(referenceDate, baseZeroRate, observationLag, frequency, indexIsInterpolated,
-                             calendar, dayCounter, seasonality) {
-    }
+    : InflationTermStructure(referenceDate, baseZeroRate, observationLag, frequency,
+                             calendar, dayCounter, seasonality) {}
 
     ZeroInflationTermStructure::ZeroInflationTermStructure(
                                     Natural settlementDays,
@@ -135,11 +214,11 @@ namespace QuantLib {
                                     Rate baseZeroRate,
                                     const Period& observationLag,
                                     Frequency frequency,
-                                    bool indexIsInterpolated,
                                     const ext::shared_ptr<Seasonality> &seasonality)
-    : InflationTermStructure(settlementDays, calendar, baseZeroRate, observationLag, frequency, indexIsInterpolated,
-                             dayCounter, seasonality) {
-    }
+    : InflationTermStructure(settlementDays, calendar, baseZeroRate, observationLag, frequency,
+                             dayCounter, seasonality) {}
+
+    QL_DEPRECATED_ENABLE_WARNING
 
     Rate ZeroInflationTermStructure::zeroRate(const Date &d, const Period& instObsLag,
                                               bool forceLinearInterpolation,
@@ -147,7 +226,7 @@ namespace QuantLib {
 
         Period useLag = instObsLag;
         if (instObsLag == Period(-1,Days)) {
-            useLag = observationLag();
+            useLag = hasExplicitBaseDate() ? Period(0, Days) : observationLag();
         }
 
         Rate zeroRate;
@@ -165,16 +244,10 @@ namespace QuantLib {
             Rate z2 = zeroRateImpl(t2);
             zeroRate = z1 + (z2-z1) * (dt/dp);
         } else {
-            if (indexIsInterpolated()) {
-                InflationTermStructure::checkRange(d-useLag, extrapolate);
-                Time t = timeFromReference(d-useLag);
-                zeroRate = zeroRateImpl(t);
-            } else {
-                std::pair<Date,Date> dd = inflationPeriod(d-useLag, frequency());
-                InflationTermStructure::checkRange(dd.first, extrapolate);
-                Time t = timeFromReference(dd.first);
-                zeroRate = zeroRateImpl(t);
-            }
+            std::pair<Date,Date> dd = inflationPeriod(d-useLag, frequency());
+            InflationTermStructure::checkRange(dd.first, extrapolate);
+            Time t = timeFromReference(dd.first);
+            zeroRate = zeroRateImpl(t);
         }
 
         if (hasSeasonality()) {
@@ -190,6 +263,43 @@ namespace QuantLib {
     }
 
 
+    YoYInflationTermStructure::YoYInflationTermStructure(
+                                    Date baseDate,
+                                    Rate baseYoYRate,
+                                    Frequency frequency,
+                                    bool indexIsInterpolated,
+                                    const DayCounter& dayCounter,
+                                    const ext::shared_ptr<Seasonality> &seasonality)
+    : InflationTermStructure(baseDate, frequency, dayCounter, seasonality, baseYoYRate),
+      indexIsInterpolated_(indexIsInterpolated) {}
+
+    YoYInflationTermStructure::YoYInflationTermStructure(
+                                    const Date& referenceDate,
+                                    Date baseDate,
+                                    Rate baseYoYRate,
+                                    Frequency frequency,
+                                    bool indexIsInterpolated,
+                                    const DayCounter& dayCounter,
+                                    const ext::shared_ptr<Seasonality> &seasonality)
+    : InflationTermStructure(referenceDate, baseDate, frequency,
+                             dayCounter, seasonality, baseYoYRate),
+      indexIsInterpolated_(indexIsInterpolated) {}
+
+    YoYInflationTermStructure::YoYInflationTermStructure(
+                                    Natural settlementDays,
+                                    const Calendar& calendar,
+                                    Date baseDate,
+                                    Rate baseYoYRate,
+                                    Frequency frequency,
+                                    bool indexIsInterpolated,
+                                    const DayCounter& dayCounter,
+                                    const ext::shared_ptr<Seasonality> &seasonality)
+    : InflationTermStructure(settlementDays, calendar, baseDate, frequency,
+                             dayCounter, seasonality, baseYoYRate),
+      indexIsInterpolated_(indexIsInterpolated) {}
+
+    QL_DEPRECATED_DISABLE_WARNING
+
     YoYInflationTermStructure::YoYInflationTermStructure(
                                     const DayCounter& dayCounter,
                                     Rate baseYoYRate,
@@ -197,8 +307,9 @@ namespace QuantLib {
                                     Frequency frequency,
                                     bool indexIsInterpolated,
                                     const ext::shared_ptr<Seasonality> &seasonality)
-    : InflationTermStructure(baseYoYRate, observationLag, frequency, indexIsInterpolated,
-                             dayCounter, seasonality) {}
+    : InflationTermStructure(baseYoYRate, observationLag, frequency,
+                             dayCounter, seasonality),
+      indexIsInterpolated_(indexIsInterpolated) {}
 
     YoYInflationTermStructure::YoYInflationTermStructure(
                                     const Date& referenceDate,
@@ -209,8 +320,9 @@ namespace QuantLib {
                                     Frequency frequency,
                                     bool indexIsInterpolated,
                                     const ext::shared_ptr<Seasonality> &seasonality)
-    : InflationTermStructure(referenceDate, baseYoYRate, observationLag, frequency, indexIsInterpolated,
-                             calendar, dayCounter, seasonality) {}
+    : InflationTermStructure(referenceDate, baseYoYRate, observationLag, frequency,
+                             calendar, dayCounter, seasonality),
+      indexIsInterpolated_(indexIsInterpolated) {}
 
     YoYInflationTermStructure::YoYInflationTermStructure(
                                     Natural settlementDays,
@@ -222,7 +334,10 @@ namespace QuantLib {
                                     bool indexIsInterpolated,
                                     const ext::shared_ptr<Seasonality> &seasonality)
     : InflationTermStructure(settlementDays, calendar, baseYoYRate, observationLag,
-                             frequency, indexIsInterpolated, dayCounter, seasonality) {}
+                             frequency, dayCounter, seasonality),
+      indexIsInterpolated_(indexIsInterpolated) {}
+
+    QL_DEPRECATED_ENABLE_WARNING
 
 
     Rate YoYInflationTermStructure::yoyRate(const Date &d, const Period& instObsLag,
@@ -231,7 +346,7 @@ namespace QuantLib {
 
         Period useLag = instObsLag;
         if (instObsLag == Period(-1,Days)) {
-            useLag = observationLag();
+            useLag = hasExplicitBaseDate() ? Period(0, Days) : observationLag();
         }
 
         Rate yoyRate;
