@@ -18,7 +18,7 @@
  FOR A PARTICULAR PURPOSE.  See the license for more details.
 */
 
-#include "overnightindexedswap.hpp"
+#include "toplevelfixture.hpp"
 #include "utilities.hpp"
 
 #include <ql/termstructures/yield/oisratehelper.hpp>
@@ -34,6 +34,7 @@
 #include <ql/time/daycounters/simpledaycounter.hpp>
 #include <ql/time/schedule.hpp>
 #include <ql/indexes/ibor/eonia.hpp>
+#include <ql/indexes/ibor/estr.hpp>
 #include <ql/indexes/ibor/euribor.hpp>
 #include <ql/indexes/ibor/fedfunds.hpp>
 #include <ql/cashflows/iborcoupon.hpp>
@@ -41,6 +42,7 @@
 #include <ql/cashflows/cashflows.hpp>
 #include <ql/cashflows/couponpricer.hpp>
 #include <ql/currencies/europe.hpp>
+#include <ql/time/calendars/unitedstates.hpp>
 #include <ql/utilities/dataformatters.hpp>
 
 #include <iostream>
@@ -51,136 +53,226 @@ using namespace boost::unit_test_framework;
 
 using std::exp;
 
-namespace overnight_indexed_swap_test {
+BOOST_FIXTURE_TEST_SUITE(QuantLibTests, TopLevelFixture)
 
-    struct Datum {
-        Integer settlementDays;
-        Integer n;
-        TimeUnit unit;
-        Rate rate;
-    };
+BOOST_AUTO_TEST_SUITE(OvernightIndexedSwapTests)
 
-    Datum depositData[] = {
-        { 0, 1, Days, 1.10 },
-        { 1, 1, Days, 1.10 },
-        { 2, 1, Weeks, 1.40 },
-        { 2, 2, Weeks, 1.50 },
-        { 2, 1, Months, 1.70 },
-        { 2, 2, Months, 1.90 },
-        { 2, 3, Months, 2.05 },
-        { 2, 4, Months, 2.08 },
-        { 2, 5, Months, 2.11 },
-        { 2, 6, Months, 2.13 }
-    };
+struct Datum {
+    Integer settlementDays;
+    Integer n;
+    TimeUnit unit;
+    Rate rate;
+};
 
-    Datum eoniaSwapData[] = {
-        { 2,  1, Weeks, 1.245 },
-        { 2,  2, Weeks, 1.269 },
-        { 2,  3, Weeks, 1.277 },
-        { 2,  1, Months, 1.281 },
-        { 2,  2, Months, 1.18 },
-        { 2,  3, Months, 1.143 },
-        { 2,  4, Months, 1.125 },
-        { 2,  5, Months, 1.116 },
-        { 2,  6, Months, 1.111 },
-        { 2,  7, Months, 1.109 },
-        { 2,  8, Months, 1.111 },
-        { 2,  9, Months, 1.117 },
-        { 2, 10, Months, 1.129 },
-        { 2, 11, Months, 1.141 },
-        { 2, 12, Months, 1.153 },
-        { 2, 15, Months, 1.218 },
-        { 2, 18, Months, 1.308 },
-        { 2, 21, Months, 1.407 },
-        { 2,  2,  Years, 1.510 },
-        { 2,  3,  Years, 1.916 },
-        { 2,  4,  Years, 2.254 },
-        { 2,  5,  Years, 2.523 },
-        { 2,  6,  Years, 2.746 },
-        { 2,  7,  Years, 2.934 },
-        { 2,  8,  Years, 3.092 },
-        { 2,  9,  Years, 3.231 },
-        { 2, 10,  Years, 3.380 },
-        { 2, 11,  Years, 3.457 },
-        { 2, 12,  Years, 3.544 },
-        { 2, 15,  Years, 3.702 },
-        { 2, 20,  Years, 3.703 },
-        { 2, 25,  Years, 3.541 },
-        { 2, 30,  Years, 3.369 }
-    };
+Datum depositData[] = {
+    { 0, 1, Days, 1.10 },
+    { 1, 1, Days, 1.10 },
+    { 2, 1, Weeks, 1.40 },
+    { 2, 2, Weeks, 1.50 },
+    { 2, 1, Months, 1.70 },
+    { 2, 2, Months, 1.90 },
+    { 2, 3, Months, 2.05 },
+    { 2, 4, Months, 2.08 },
+    { 2, 5, Months, 2.11 },
+    { 2, 6, Months, 2.13 }
+};
 
-    struct CommonVars {
-        // global data
-        Date today, settlement;
-        Swap::Type type;
-        Real nominal;
-        Calendar calendar;
-        Natural settlementDays;
+Datum eoniaSwapData[] = {
+    { 2,  1, Weeks, 1.245 },
+    { 2,  2, Weeks, 1.269 },
+    { 2,  3, Weeks, 1.277 },
+    { 2,  1, Months, 1.281 },
+    { 2,  2, Months, 1.18 },
+    { 2,  3, Months, 1.143 },
+    { 2,  4, Months, 1.125 },
+    { 2,  5, Months, 1.116 },
+    { 2,  6, Months, 1.111 },
+    { 2,  7, Months, 1.109 },
+    { 2,  8, Months, 1.111 },
+    { 2,  9, Months, 1.117 },
+    { 2, 10, Months, 1.129 },
+    { 2, 11, Months, 1.141 },
+    { 2, 12, Months, 1.153 },
+    { 2, 15, Months, 1.218 },
+    { 2, 18, Months, 1.308 },
+    { 2, 21, Months, 1.407 },
+    { 2,  2,  Years, 1.510 },
+    { 2,  3,  Years, 1.916 },
+    { 2,  4,  Years, 2.254 },
+    { 2,  5,  Years, 2.523 },
+    { 2,  6,  Years, 2.746 },
+    { 2,  7,  Years, 2.934 },
+    { 2,  8,  Years, 3.092 },
+    { 2,  9,  Years, 3.231 },
+    { 2, 10,  Years, 3.380 },
+    { 2, 11,  Years, 3.457 },
+    { 2, 12,  Years, 3.544 },
+    { 2, 15,  Years, 3.702 },
+    { 2, 20,  Years, 3.703 },
+    { 2, 25,  Years, 3.541 },
+    { 2, 30,  Years, 3.369 }
+};
 
-        Period fixedEoniaPeriod, floatingEoniaPeriod;
-        DayCounter fixedEoniaDayCount;
-        BusinessDayConvention fixedEoniaConvention, floatingEoniaConvention;
-        ext::shared_ptr<Eonia> eoniaIndex;
-        RelinkableHandle<YieldTermStructure> eoniaTermStructure;
+struct CommonVars {
+    // global data
+    Date today, settlement;
+    Swap::Type type;
+    Real nominal;
+    Calendar calendar;
+    Natural settlementDays;
 
-        Frequency fixedSwapFrequency;
-        DayCounter fixedSwapDayCount;
-        BusinessDayConvention fixedSwapConvention;
-        ext::shared_ptr<IborIndex> swapIndex;
-        RelinkableHandle<YieldTermStructure> swapTermStructure;
+    Period fixedEoniaPeriod, floatingEoniaPeriod;
+    DayCounter fixedEoniaDayCount;
+    BusinessDayConvention fixedEoniaConvention, floatingEoniaConvention;
+    ext::shared_ptr<Eonia> eoniaIndex;
+    RelinkableHandle<YieldTermStructure> eoniaTermStructure;
 
-        // utilities
-        ext::shared_ptr<OvernightIndexedSwap>
-        makeSwap(Period length,
-                 Rate fixedRate,
-                 Spread spread,
-                 bool telescopicValueDates,
-                 Date effectiveDate = Null<Date>(),
-                 Natural paymentLag = 0,
-                 RateAveraging::Type averagingMethod = RateAveraging::Compound) {
-            return MakeOIS(length, eoniaIndex, fixedRate, 0 * Days)
-                .withEffectiveDate(effectiveDate == Null<Date>() ? settlement : effectiveDate)
-                .withOvernightLegSpread(spread)
-                .withNominal(nominal)
-                .withPaymentLag(paymentLag)
-                .withDiscountingTermStructure(eoniaTermStructure)
-                .withTelescopicValueDates(telescopicValueDates)
-                .withAveragingMethod(averagingMethod);
-        }
+    Frequency fixedSwapFrequency;
+    DayCounter fixedSwapDayCount;
+    BusinessDayConvention fixedSwapConvention;
+    ext::shared_ptr<IborIndex> swapIndex;
+    RelinkableHandle<YieldTermStructure> swapTermStructure;
 
-        CommonVars() {
-            type = Swap::Payer;
-            settlementDays = 2;
-            nominal = 100.0;
-            fixedEoniaConvention = ModifiedFollowing;
-            floatingEoniaConvention = ModifiedFollowing;
-            fixedEoniaPeriod = 1*Years;
-            floatingEoniaPeriod = 1*Years;
-            fixedEoniaDayCount = Actual360();
-            eoniaIndex = ext::make_shared<Eonia>(eoniaTermStructure);
-            fixedSwapConvention = ModifiedFollowing;
-            fixedSwapFrequency = Annual;
-            fixedSwapDayCount = Thirty360(Thirty360::BondBasis);
-            swapIndex = ext::shared_ptr<IborIndex>(new Euribor3M(swapTermStructure));
-            calendar = eoniaIndex->fixingCalendar();
-            today = Date(5, February, 2009);
-            //today = calendar.adjust(Date::todaysDate());
-            Settings::instance().evaluationDate() = today;
-            settlement = calendar.advance(today,settlementDays*Days,Following);
-            eoniaTermStructure.linkTo(flatRate(today, 0.05,
-                                               Actual365Fixed()));
-        }
-    };
+    // utilities
+    ext::shared_ptr<OvernightIndexedSwap>
+    makeSwap(Period length,
+             Rate fixedRate,
+             Spread spread,
+             bool telescopicValueDates,
+             Date effectiveDate = Null<Date>(),
+             Integer paymentLag = 0,
+             RateAveraging::Type averagingMethod = RateAveraging::Compound) {
+        return MakeOIS(length, eoniaIndex, fixedRate, 0 * Days)
+            .withEffectiveDate(effectiveDate == Null<Date>() ? settlement : effectiveDate)
+            .withOvernightLegSpread(spread)
+            .withNominal(nominal)
+            .withPaymentLag(paymentLag)
+            .withDiscountingTermStructure(eoniaTermStructure)
+            .withTelescopicValueDates(telescopicValueDates)
+            .withAveragingMethod(averagingMethod);
+    }
 
+    ext::shared_ptr<OvernightIndexedSwap>
+    makeSwapWithLookback(Period length,
+                         Rate fixedRate,
+                         Integer paymentLag,
+                         Natural lookbackDays,
+                         Natural lockoutDays,
+                         bool applyObservationShift,
+                         bool telescopicValueDates) {
+        return MakeOIS(length, eoniaIndex, fixedRate, 0 * Days)
+            .withEffectiveDate(settlement)
+            .withNominal(nominal)
+            .withPaymentLag(paymentLag)
+            .withDiscountingTermStructure(eoniaTermStructure)
+            .withLookbackDays(lookbackDays)
+            .withLockoutDays(lockoutDays)
+            .withObservationShift(applyObservationShift)
+            .withTelescopicValueDates(telescopicValueDates);
+    }
+
+    CommonVars() {
+        type = Swap::Payer;
+        settlementDays = 2;
+        nominal = 100.0;
+        fixedEoniaConvention = ModifiedFollowing;
+        floatingEoniaConvention = ModifiedFollowing;
+        fixedEoniaPeriod = 1*Years;
+        floatingEoniaPeriod = 1*Years;
+        fixedEoniaDayCount = Actual360();
+        eoniaIndex = ext::make_shared<Eonia>(eoniaTermStructure);
+        fixedSwapConvention = ModifiedFollowing;
+        fixedSwapFrequency = Annual;
+        fixedSwapDayCount = Thirty360(Thirty360::BondBasis);
+        swapIndex = ext::shared_ptr<IborIndex>(new Euribor3M(swapTermStructure));
+        calendar = eoniaIndex->fixingCalendar();
+        today = Date(5, February, 2009);
+        //today = calendar.adjust(Date::todaysDate());
+        Settings::instance().evaluationDate() = today;
+        settlement = calendar.advance(today,settlementDays*Days,Following);
+        eoniaTermStructure.linkTo(flatRate(today, 0.05,
+                                           Actual365Fixed()));
+    }
+};
+
+void testBootstrap(bool telescopicValueDates,
+                   RateAveraging::Type averagingMethod,
+                   Real tolerance = 1.0e-8) {
+
+    CommonVars vars;
+
+    Natural paymentLag = 2;
+
+    std::vector<ext::shared_ptr<RateHelper> > eoniaHelpers;
+
+    ext::shared_ptr<IborIndex> euribor3m(new Euribor3M);
+    ext::shared_ptr<Eonia> eonia(new Eonia);
+
+    for (auto& i : depositData) {
+        Real rate = 0.01 * i.rate;
+        ext::shared_ptr<SimpleQuote> simple = ext::make_shared<SimpleQuote>(rate);
+        ext::shared_ptr<Quote> quote (simple);
+        Period term = i.n * i.unit;
+        ext::shared_ptr<RateHelper> helper(new DepositRateHelper(
+                Handle<Quote>(quote), term, i.settlementDays, euribor3m->fixingCalendar(),
+                euribor3m->businessDayConvention(), euribor3m->endOfMonth(), euribor3m->dayCounter()));
+
+        if (term <= 2*Days)
+            eoniaHelpers.push_back(helper);
+    }
+
+    for (auto& i : eoniaSwapData) {
+        Real rate = 0.01 * i.rate;
+        ext::shared_ptr<SimpleQuote> simple = ext::make_shared<SimpleQuote>(rate);
+        ext::shared_ptr<Quote> quote (simple);
+        Period term = i.n * i.unit;
+        ext::shared_ptr<RateHelper> helper(new OISRateHelper(i.settlementDays,
+                                                             term,
+                                                             Handle<Quote>(quote),
+                                                             eonia,
+                                                             Handle<YieldTermStructure>(),
+                                                             telescopicValueDates,
+                                                             paymentLag,
+                                                             Following,
+                                                             Annual,
+                                                             Calendar(),
+                                                             0 * Days,
+                                                             0.0,
+                                                             Pillar::LastRelevantDate,
+                                                             Date(),
+                                                             averagingMethod));
+        eoniaHelpers.push_back(helper);
+    }
+
+    auto eoniaTS = ext::make_shared<PiecewiseYieldCurve<Discount, LogLinear>>(vars.today, eoniaHelpers, Actual365Fixed());
+
+    vars.eoniaTermStructure.linkTo(eoniaTS);
+
+    // test curve consistency
+    for (auto& i : eoniaSwapData) {
+        Rate expected = i.rate / 100;
+        Period term = i.n * i.unit;
+        // test telescopic value dates (in bootstrap) against non telescopic value dates (swap here)
+        ext::shared_ptr<OvernightIndexedSwap> swap =
+            vars.makeSwap(term, 0.0, 0.0, false, Null<Date>(), paymentLag, averagingMethod);
+        Rate calculated = swap->fairRate();
+        Rate error = std::fabs(expected-calculated);
+
+        if (error>tolerance)
+            BOOST_FAIL("curve inconsistency:" << std::setprecision(10) <<
+                       "\n swap length:     " << term <<
+                       "\n quoted rate:     " << expected <<
+                       "\n calculated rate: " << calculated <<
+                       "\n error:           " << error <<
+                       "\n tolerance:       " << tolerance);
+    }
 }
 
 
-void OvernightIndexedSwapTest::testFairRate() {
+BOOST_AUTO_TEST_CASE(testFairRate) {
 
     BOOST_TEST_MESSAGE("Testing Eonia-swap calculation of fair fixed rate...");
 
-    using namespace overnight_indexed_swap_test;
-
     CommonVars vars;
 
     Period lengths[] = { 1*Years, 2*Years, 5*Years, 10*Years, 20*Years };
@@ -218,14 +310,11 @@ void OvernightIndexedSwapTest::testFairRate() {
     }
 }
 
-
-void OvernightIndexedSwapTest::testFairSpread() {
+BOOST_AUTO_TEST_CASE(testFairSpread) {
 
     BOOST_TEST_MESSAGE("Testing Eonia-swap calculation of "
                        "fair floating spread...");
 
-    using namespace overnight_indexed_swap_test;
-
     CommonVars vars;
 
     Period lengths[] = { 1*Years, 2*Years, 5*Years, 10*Years, 20*Years };
@@ -264,12 +353,10 @@ void OvernightIndexedSwapTest::testFairSpread() {
     }
 }
 
-void OvernightIndexedSwapTest::testCachedValue() {
+BOOST_AUTO_TEST_CASE(testCachedValue) {
 
     BOOST_TEST_MESSAGE("Testing Eonia-swap calculation against cached value...");
 
-    using namespace overnight_indexed_swap_test;
-
     CommonVars vars;
 
     Settings::instance().evaluationDate() = vars.today;
@@ -296,58 +383,73 @@ void OvernightIndexedSwapTest::testCachedValue() {
                     "\n tolerance:" << tolerance);
 }
 
-namespace overnight_indexed_swap_test {
-    void testBootstrap(bool telescopicValueDates,
-                       RateAveraging::Type averagingMethod,
-                       Real tolerance = 1.0e-8) {
+BOOST_AUTO_TEST_CASE(testBaseBootstrap) {
+    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with daily compounded ON rates...");
+    testBootstrap(false, RateAveraging::Compound);
+}
+
+BOOST_AUTO_TEST_CASE(testBootstrapWithArithmeticAverage) {
+    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with arithmetic average ON rates...");
+    testBootstrap(false, RateAveraging::Simple);
+}
+
+BOOST_AUTO_TEST_CASE(testBootstrapWithTelescopicDates) {
+    BOOST_TEST_MESSAGE(
+        "Testing Eonia-swap curve building with telescopic value dates and DCON rates...");
+    testBootstrap(true, RateAveraging::Compound);
+}
+
+BOOST_AUTO_TEST_CASE(testBootstrapWithTelescopicDatesAndArithmeticAverage) {
+    BOOST_TEST_MESSAGE(
+        "Testing Eonia-swap curve building with telescopic value dates and AAON rates...");
+    // Given that we are using an approximation that omits
+    // the required convexity correction, a lower tolerance
+    // is needed.
+    testBootstrap(true, RateAveraging::Simple, 1.0e-5);
+}
+
+
+void testBootstrapWithLookback(Natural lookbackDays,
+                               Natural lockoutDays,
+                               bool applyObservationShift,
+                               bool telescopicValueDates,
+                               Natural paymentLag) {
 
     CommonVars vars;
 
-    Natural paymentLag = 2;
-
     std::vector<ext::shared_ptr<RateHelper> > eoniaHelpers;
 
-    ext::shared_ptr<IborIndex> euribor3m(new Euribor3M);
     ext::shared_ptr<Eonia> eonia(new Eonia);
 
-    for (auto& i : depositData) {
-        Real rate = 0.01 * i.rate;
-        ext::shared_ptr<SimpleQuote> simple = ext::make_shared<SimpleQuote>(rate);
-        ext::shared_ptr<Quote> quote (simple);
-        Period term = i.n * i.unit;
-        ext::shared_ptr<RateHelper> helper(new DepositRateHelper(
-            Handle<Quote>(quote), term, i.settlementDays, euribor3m->fixingCalendar(),
-            euribor3m->businessDayConvention(), euribor3m->endOfMonth(), euribor3m->dayCounter()));
-
-        if (term <= 2*Days)
-            eoniaHelpers.push_back(helper);
-    }
-
     for (auto& i : eoniaSwapData) {
         Real rate = 0.01 * i.rate;
-        ext::shared_ptr<SimpleQuote> simple = ext::make_shared<SimpleQuote>(rate);
-        ext::shared_ptr<Quote> quote (simple);
+        auto quote = ext::make_shared<SimpleQuote>(rate);
         Period term = i.n * i.unit;
-        ext::shared_ptr<RateHelper> helper(new
-                     OISRateHelper(i.settlementDays,
-                                   term,
-                                   Handle<Quote>(quote),
-                                   eonia,
-                                   Handle<YieldTermStructure>(),
-                                   telescopicValueDates,
-                                   paymentLag, 
-                                   Following, 
-                                   Annual, 
-                                   Calendar(), 
-                                   0 * Days, 
-                                   0.0, 
-                                   Pillar::LastRelevantDate, 
-                                   Date(), 
-                                   averagingMethod));
+        auto helper = ext::make_shared<OISRateHelper>(i.settlementDays,
+                                                      term,
+                                                      Handle<Quote>(quote),
+                                                      eonia,
+                                                      Handle<YieldTermStructure>(),
+                                                      telescopicValueDates,
+                                                      paymentLag,
+                                                      Following,
+                                                      Annual,
+                                                      Calendar(),
+                                                      0 * Days,
+                                                      0.0,
+                                                      Pillar::LastRelevantDate,
+                                                      Date(),
+                                                      RateAveraging::Compound,
+                                                      ext::nullopt,
+                                                      ext::nullopt,
+                                                      Calendar(),
+                                                      lookbackDays,
+                                                      lockoutDays,
+                                                      applyObservationShift);
         eoniaHelpers.push_back(helper);
     }
 
-    auto eoniaTS = ext::make_shared<PiecewiseYieldCurve<Discount, LogLinear>>(vars.today, eoniaHelpers, Actual365Fixed());
+    auto eoniaTS = ext::make_shared<PiecewiseYieldCurve<ForwardRate, BackwardFlat>>(vars.today, eoniaHelpers, Actual365Fixed());
 
     vars.eoniaTermStructure.linkTo(eoniaTS);
 
@@ -355,54 +457,86 @@ namespace overnight_indexed_swap_test {
     for (auto& i : eoniaSwapData) {
         Rate expected = i.rate / 100;
         Period term = i.n * i.unit;
-        // test telescopic value dates (in bootstrap) against non telescopic value dates (swap here)
         ext::shared_ptr<OvernightIndexedSwap> swap =
-            vars.makeSwap(term, 0.0, 0.0, false, Null<Date>(), paymentLag, averagingMethod);
+            vars.makeSwapWithLookback(term, 0.0, paymentLag, lookbackDays, lockoutDays,
+                                      applyObservationShift, telescopicValueDates);
         Rate calculated = swap->fairRate();
         Rate error = std::fabs(expected-calculated);
+        Real tolerance = 1e-8;
 
         if (error>tolerance)
             BOOST_FAIL("curve inconsistency:" << std::setprecision(10) <<
-                        "\n swap length:     " << term <<
-                        "\n quoted rate:     " << expected <<
-                        "\n calculated rate: " << calculated <<
-                        "\n error:           " << error <<
-                        "\n tolerance:       " << tolerance);
+                       "\n swap length:     " << term <<
+                       "\n quoted rate:     " << expected <<
+                       "\n calculated rate: " << calculated <<
+                       "\n error:           " << error <<
+                       "\n tolerance:       " << tolerance);
     }
-} // testBootstrap(telescopicValueDates)
-} // anonymous namespace
-
-void OvernightIndexedSwapTest::testBootstrap() {
-    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with daily compounded ON rates...");
-    overnight_indexed_swap_test::testBootstrap(false, RateAveraging::Compound);
 }
 
-void OvernightIndexedSwapTest::testBootstrapWithArithmeticAverage() {
-    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with arithmetic average ON rates...");
-    overnight_indexed_swap_test::testBootstrap(false, RateAveraging::Simple);
+
+
+BOOST_AUTO_TEST_CASE(testBootstrapWithLookbackDays) {
+    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with lookback days...");
+
+    auto lookbackDays = 2;
+    auto lockoutDays = 0;
+    auto applyObservationShift = false;
+    auto paymentLag = 2;
+
+    testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, false, paymentLag);
+
+    BOOST_CHECK_EXCEPTION(
+        testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, true, paymentLag),
+        Error, ExpectedErrorMessage("Telescopic formula cannot be applied"));
 }
 
-void OvernightIndexedSwapTest::testBootstrapWithTelescopicDates() {
-    BOOST_TEST_MESSAGE(
-        "Testing Eonia-swap curve building with telescopic value dates and DCON rates...");
-    overnight_indexed_swap_test::testBootstrap(true, RateAveraging::Compound);
+BOOST_AUTO_TEST_CASE(testBootstrapWithLookbackDaysAndShift) {
+    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with lookback days and observation shift...");
+
+    auto lookbackDays = 2;
+    auto lockoutDays = 0;
+    auto applyObservationShift = true;
+    auto paymentLag = 2;
+
+    testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, false, paymentLag);
+
+    testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, true, paymentLag);
 }
 
-void OvernightIndexedSwapTest::testBootstrapWithTelescopicDatesAndArithmeticAverage() {
-    BOOST_TEST_MESSAGE(
-        "Testing Eonia-swap curve building with telescopic value dates and AAON rates...");
-    // Given that we are using an approximation that omits
-    // the required convexity correction, a lower tolerance
-    // is needed.
-    overnight_indexed_swap_test::testBootstrap(true, RateAveraging::Simple, 1.0e-5);
+BOOST_AUTO_TEST_CASE(testBootstrapWithLockoutDays) {
+    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with lookback and lockout days...");
+
+    auto lookbackDays = 2;
+    auto lockoutDays = 2;
+    auto applyObservationShift = false;
+    auto paymentLag = 0;
+
+    testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, false, paymentLag);
+
+    BOOST_CHECK_EXCEPTION(
+        testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, true, paymentLag),
+        Error, ExpectedErrorMessage("Telescopic formula cannot be applied"));
 }
 
-void OvernightIndexedSwapTest::testSeasonedSwaps() {
+BOOST_AUTO_TEST_CASE(testBootstrapWithLockoutDaysAndShift) {
+    BOOST_TEST_MESSAGE("Testing Eonia-swap curve building with lookback and lockout days and observation shift...");
+
+    auto lookbackDays = 2;
+    auto lockoutDays = 2;
+    auto applyObservationShift = true;
+    auto paymentLag = 0;
+
+    testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, false, paymentLag);
+
+    testBootstrapWithLookback(lookbackDays, lockoutDays, applyObservationShift, true, paymentLag);
+}
+
+
+BOOST_AUTO_TEST_CASE(testSeasonedSwaps) {
 
     BOOST_TEST_MESSAGE("Testing seasoned Eonia-swap calculation...");
 
-    using namespace overnight_indexed_swap_test;
-
     CommonVars vars;
 
     Period lengths[] = { 1*Years, 2*Years, 5*Years, 10*Years, 20*Years };
@@ -433,12 +567,9 @@ void OvernightIndexedSwapTest::testSeasonedSwaps() {
     }
 }
 
-
-void OvernightIndexedSwapTest::testBootstrapRegression() {
+BOOST_AUTO_TEST_CASE(testBootstrapRegression) {
     BOOST_TEST_MESSAGE("Testing 1.16 regression with OIS bootstrap...");
 
-    using namespace overnight_indexed_swap_test;
-
     Datum data[] = {
         { 0,  1, Days,   0.0066   },
         { 2,  1, Weeks,  0.006445 },
@@ -499,8 +630,7 @@ void OvernightIndexedSwapTest::testBootstrapRegression() {
     BOOST_CHECK_NO_THROW(curve.discount(1.0));
 }
 
-
-void OvernightIndexedSwapTest::test131BootstrapRegression() {
+BOOST_AUTO_TEST_CASE(test131BootstrapRegression) {
     BOOST_TEST_MESSAGE("Testing 1.31 regression with OIS bootstrap...");
 
     Date today(11, December, 2012);
@@ -516,11 +646,9 @@ void OvernightIndexedSwapTest::test131BootstrapRegression() {
     BOOST_CHECK_NO_THROW(curve.nodes());
 }
 
-void OvernightIndexedSwapTest::testConstructorsAndNominals() {
+BOOST_AUTO_TEST_CASE(testConstructorsAndNominals) {
     BOOST_TEST_MESSAGE("Testing different constructors for OIS...");
 
-    using namespace overnight_indexed_swap_test;
-
     CommonVars vars;
 
     Date spot = vars.calendar.advance(vars.today, 2*Days);
@@ -648,19 +776,47 @@ void OvernightIndexedSwapTest::testConstructorsAndNominals() {
     BOOST_CHECK_EQUAL(ois_4.overnightNominals()[3], nominal/2);
 }
 
+BOOST_AUTO_TEST_CASE(testNotifications) {
+    BOOST_TEST_MESSAGE("Testing cash-flow notifications for OIS...");
 
-test_suite* OvernightIndexedSwapTest::suite() {
-    auto* suite = BOOST_TEST_SUITE("Overnight-indexed swap tests");
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testFairRate));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testFairSpread));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testCachedValue));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testBootstrap));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testBootstrapWithArithmeticAverage));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testBootstrapWithTelescopicDates));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testBootstrapWithTelescopicDatesAndArithmeticAverage));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testSeasonedSwaps));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testBootstrapRegression));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::test131BootstrapRegression));
-    suite->add(QUANTLIB_TEST_CASE(&OvernightIndexedSwapTest::testConstructorsAndNominals));
-    return suite;
+    CommonVars vars;
+
+    Date spot = vars.calendar.advance(vars.today, 2*Days);
+    Real nominal = 100000.0;
+
+    Schedule schedule = MakeSchedule()
+        .from(spot)
+        .to(vars.calendar.advance(spot, 2*Years))
+        .withCalendar(vars.calendar)
+        .withFrequency(Annual);
+
+    RelinkableHandle<YieldTermStructure> forecast_handle;
+    forecast_handle.linkTo(flatRate(0.02, Actual360()));
+
+    RelinkableHandle<YieldTermStructure> discount_handle;
+    discount_handle.linkTo(flatRate(0.02, Actual360()));
+    
+    auto index = ext::make_shared<Estr>(forecast_handle);
+    
+    auto ois = ext::make_shared<OvernightIndexedSwap>(Swap::Payer,
+                                                      nominal,
+                                                      schedule,
+                                                      0.03,
+                                                      Actual360(),
+                                                      index);
+    ois->setPricingEngine(ext::make_shared<DiscountingSwapEngine>(discount_handle));
+    ois->NPV();
+
+    Flag flag;
+    flag.registerWith(ois);
+    flag.lower();
+
+    forecast_handle.linkTo(flatRate(0.03, Actual360()));
+
+    if (!flag.isUp())
+        BOOST_FAIL("OIS was not notified of curve change");
 }
+
+BOOST_AUTO_TEST_SUITE_END()
+
+BOOST_AUTO_TEST_SUITE_END()
